from ...app import Kard
from ... import kard

class KardAccount:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidMe { me { ... Me_MeParts }}\n\n"\
            "fragment Card_CardParts on Card { __typename id activatedAt customText name visibleNumber blocked expirationAt "\
                "scheme ... on PhysicalCard { atm contactless swipe online design reorderExpiredFees { value currency { isoCode } } "\
                    "orderedAt orderDelayed isMain isSoonExpired estimatedCraftingDate estimatedDeliveryDate "\
                "}"\
            "}\n\n"\
            "fragment Topup_TopupCardParts on TopupCard { "\
                "id name expirationDate default last4 providerSourceId providerPaymentId"\
            "}\n\n"\
            "fragment Me_SubscriptionParts on Subscription { "\
                "id status cancelledAt cancellationReason maxChildren maxParents hasInsurance promotionalCreditBalance { "\
                    "value currency { isoCode name symbol symbolFirst } "\
                "} nextBilling { date amount { value currency { isoCode name symbol symbolFirst } } } "\
                    "plan { __typename id periodUnit name providerId price { value } }"\
            "}\n\n"\
            "fragment Topup_RecurringParts on RecurringPayment { "\
                "id active amount { value currency { symbol isoCode } } child { id } firstPayment nextPayment cancelledAt "\
                    "topupCard { ... Topup_TopupCardParts }"\
                "}\n\n"\
            "fragment Me_TopupRequestParts on TopupRequest { "\
                "id amount { value currency { symbol isoCode } } reason accepted cancelled declined createdAt "\
                    "requestee { id nickname firstName lastName avatar { url } } "\
                    "requester { id nickname firstName lastName avatar { url } }"\
            "}\n\n"\
            "fragment Transaction_TransactionCashbackParts on Transaction { "\
                "id title amount { value currency { symbol } } image { id url } processedAt category { name color image { url } }"\
                " ... on CashbackTransaction { cashback { status brandLogo brandName sourceTransaction { "\
                    "id title image { id url } category { name color image { url } } amount { value currency { symbol } } } } }"\
            "}\n\n"\
            "fragment Me_MeParts on Me { "\
                "id modirumId type intercomHashes { android } teenOnboardingFlow parentOnboardingFlow canFinishOnboarding "\
                    "profile { avatar { url } firstName lastName username age birthday placeOfBirth "\
                        "shippingAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } "\
                        "homeAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } "\
                    "} email emailConfirmed unconfirmedEmail phoneNumber referralCode referralUrl "\
                    "bankAccount { id iban bic user { firstName lastName } balance { value currency { symbol isoCode } } "\
                        "lockedTransactions createdAt "\
                    "} card { id } cardBeingSetup { __typename id customText name ... on PhysicalCard { design } } "\
                    "cards { nodes { ... Card_CardParts } } earnings { value currency { symbol isoCode } } onboardingDone "\
                    "passcodeSet pendingDebts { amount { value currency { symbol isoCode } } id "\
                        "owner { avatar { url } firstName id lastName username } reason } "\
                    "topupCards { ... Topup_TopupCardParts } subscription { ... Me_SubscriptionParts } "\
                    "outgoingRecurringPayments { ... Topup_RecurringParts } incomingRecurringPayment { ... Topup_RecurringParts } "\
                    "fundsOrigin canOrderCard externalAuthenticationProviders { id type uniqueId } claimId cardTransactionsCount "\
                    "topupRequestsFromChildren { ... Me_TopupRequestParts } topupRequestsToParent { ... Me_TopupRequestParts } "\
                    "savingsAmount { value } createdAt cashbackEnabled cashbackWallet { "\
                        "id balance { value currency { symbol isoCode } } "\
                        "amountEarned { value currency { symbol isoCode } } "\
                        "transactions(first: 10, order: CREATED, direction: DESC) { "\
                            "pageInfo { endCursor hasNextPage } "\
                            "nodes { ... Transaction_TransactionCashbackParts } "\
                        "} "\
                    "} invitedByOther topupIncentiveActivated unreadActivityItemsCount noPhoneNumber allowedToAddChild "\
                    "modirumEnabled currentSpendings { "\
                        "monthlyPos { value } weeklyPos { value } weeklyAtm { value } monthlyAtm { value } "\
                    "} legalSpendingLimits { monthlyPos { value } weeklyPos { value } weeklyAtm { value } monthlyAtm { value } } "\
                    "transactionLimits { id amount { value currency { symbol } } recurrence transactionType } "\
                    "transactionAuthorizations { authorizationType isAuthorized } "\
                    "familyRewardAmount { value currency { isoCode } } familyRewardActivated offboardingReasonValidatedAt "\
                    "wireTransfersMinAmount { value currency { isoCode symbol } } family { accountsClosedAt } "\
                    "teenPaysForKardEnabled confirmedAdult canVoteForFeatures"\
            "}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me'):
            raise kard.exceptions.GraphQLException('Expected "me" to be present. Got: {}'.format(request))

        return request['me']

    @property
    def id(self):
        return self.getId()

    @property
    def firstName(self):
        return self.getFirstName()

    @property
    def lastName(self):
        return self.getLastName()

    @property
    def email(self):
        return self.getEmail()

    @property
    def phoneNumber(self):
        return self.getPhoneNumber()

    @property
    def referralCode(self):
        return self.getReferralCode()
    
    @property
    def referralUrl(self):
        return self.getReferralUrl()
    
    @property
    def avatar(self):
        return self.getAvatar()

    @property
    def age(self):
        return self.getAge()

    @property
    def birthday(self):
        return self.getBirthday()

    @property
    def birthDate(self):
        return self.getBirthday()

    @property
    def placeOfBirth(self):
        return self.getPlaceOfBirth()

    @property
    def birthPlace(self):
        return self.getPlaceOfBirth()

    @property
    def shippingAddress(self):
        return self.getShippingAddress()

    @property
    def homeAddress(self):
        return self.getHomeAddress()


    def getId(self):
        query = "query androidMe { me { id }}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('id'):
            raise kard.exceptions.GraphQLException('Expected "me.id" to be present. Got: {}'.format(request))

        return request['me']['id']

    def getFirstName(self):
        query = "query androidMe { me { profile { firstName }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('firstName'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.firstName" to be present. Got: {}'.format(request))

        return request['me']['profile']['firstName']

    def getLastName(self):
        query = "query androidMe { me { profile { lastName }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('lastName'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.lastName" to be present. Got: {}'.format(request))

        return request['me']['profile']['lastName']

    def getEmail(self):
        query = "query androidMe { me { email }}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('email'):
            raise kard.exceptions.GraphQLException('Expected "me.email" to be present. Got: {}'.format(request))

        return request['me']['email']
    
    def getPhoneNumber(self):
        query = "query androidMe { me { phoneNumber }}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('phoneNumber'):
            raise kard.exceptions.GraphQLException('Expected "me.phoneNumber" to be present. Got: {}'.format(request))

        return request['me']['phoneNumber']
    
    def getReferralCode(self):
        query = "query androidMe { me { referralCode }}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('referralCode'):
            raise kard.exceptions.GraphQLException('Expected "me.referralCode" to be present. Got: {}'.format(request))

        return request['me']['referralCode']
    
    def getReferralUrl(self):
        query = "query androidMe { me { referralUrl }}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('referralUrl'):
            raise kard.exceptions.GraphQLException('Expected "me.referralUrl" to be present. Got: {}'.format(request))

        return request['me']['referralUrl']
    
    def getAvatar(self):
        query = "query androidMe { me { avatar { url }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('avatar', {}).get('url'):
            raise kard.exceptions.GraphQLException('Expected "me.avatar.url" to be present. Got: {}'.format(request))

        return request['me']['avatar']['url']
    
    def getAge(self):
        query = "query androidMe { me { profile { age }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('age'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.age" to be present. Got: {}'.format(request))

        return request['me']['profile']['age']

    def getBirthday(self):
        query = "query androidMe { me { profile { birthday }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('birthday'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.birthday" to be present. Got: {}'.format(request))

        return request['me']['profile']['birthday']
    
    def getPlaceOfBirth(self):
        query = "query androidMe { me { profile { placeOfBirth }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('placeOfBirth'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.placeOfBirth" to be present. Got: {}'.format(request))

        return request['me']['profile']['placeOfBirth']
    
    def getShippingAddress(self):
        query = "query androidMe { me { profile { shippingAddress { street line1 line2 city state zipcode country fullAddress } }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('shippingAddress'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.shippingAddress" to be present. Got: {}'.format(request))

        return request['me']['profile']['shippingAddress']
    
    def getHomeAddress(self):
        query = "query androidMe { me { profile { homeAddress { street line1 line2 city state zipcode country fullAddress } }}}"
        variables = {}
        extensions = {}

        request = self.app.graphql.request(query, variables, extensions)
        if not request.get('me', {}).get('profile', {}).get('homeAddress'):
            raise kard.exceptions.GraphQLException('Expected "me.profile.homeAddress" to be present. Got: {}'.format(request))

        return request['me']['profile']['homeAddress']




from . import bankAccount
from . import beneficiaries
from . import beneficiary
from . import cards
from . import cashback
from . import contacts
from . import details
from . import family
from . import kyc
from . import notifications
from . import offers
from . import subscription
from . import transactions
from . import topupCards
from . import topupCard
from . import vaults
from . import vault

def setup(kard):
    kard.bankAccount = bankAccount.KardBank(kard)
    kard.beneficiaries = beneficiaries.KardBeneficiaries(kard)
    kard.beneficiaries.beneficiary = beneficiary.KardBeneficiary(kard)
    kard.cards = cards.KardCards(kard)
    kard.cashback = cashback.KardCashback(kard)
    kard.cashback.offers = offers.KardOffers(kard)
    kard.contacts = contacts.KardContacts(kard)
    kard.details = details.KardAccount(kard)
    kard.family = family.KardFamily(kard)
    kard.kyc = kyc.KardKYC(kard)
    kard.notifications = notifications.KardNotifications(kard)
    kard.subscription = subscription.KardSubscription(kard)
    kard.transactions = transactions.KardTransactions(kard)
    kard.topupCards = topupCards.KardTopupCards(kard)
    kard.topupCards.topupCard = topupCard.KardTopupCard(kard)
    kard.vaults = vaults.KardVaults(kard)
    kard.vaults.vault = vault.KardVault(kard)

    return kard



from ...app import Kard
from ... import kard

import requests

class KardTopupCards:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidListTopupCard { me { topupCards { ... Topup_TopupCardParts } }}\n\n"\
                "fragment Topup_TopupCardParts on TopupCard { id name expirationDate default last4 providerSourceId providerPaymentId}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('topupCards'):
            raise kard.exceptions.GraphQLException('Expected "me.topupCards" to be present. Got: ' + str(response))

        return response['me']['topupCards']

    def getAll(self):
        return self.getCompleteData()

    def get(self, cardId: str):
        cards = self.getAll()
        for card in cards:
            if card['id'] == cardId:
                return card
        
        raise kard.exceptions.CardNotFoundException('Card %s not found' % cardId)

    def getDefault(self):
        cards = self.getAll()
        for card in cards:
            if card['default']:
                return card
        
        raise kard.exceptions.CardNotFoundException('No default card found')
    

    def add(self, digits: str, exp_month: str, exp_year: str, cvv: str):
        # Step 1: Create a token
        r = requests.post(
            'https://api.checkout.com/tokens',
            json={
                'type': 'card',
                'number': digits,
                'expiry_month': exp_month,
                'expiry_year': exp_year,
                'cvv': cvv
            },
            headers={
                'authorization': 'INVESTIGATE IF THIS IS PRIVATE, IF SO, HOW IS IT OBTAINED'
            }
        )
        if r.status_code != 201:
            raise ValueError('Failed to create token: %s: %s' % (r.status_code, r.text))
        
        # Step 2: Add the token to Kard
        query = "mutation androidAddTopupCard($token: String!, $failureUrl: String, $successUrl: String) { "\
                    "addTopupCard(input: {token: $token, failureUrl: $failureUrl, successUrl: $successUrl}) { "\
                        "paymentId secureFormUrl errors { message path } "\
                    "}"\
                "}"
        variables = {
            "token": r.json()['token'],
            "failureUrl": "https://eu.kard.app/3ds/failure",
            "successUrl": "https://eu.kard.app/3ds/success"
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('addTopupCard'):
            raise kard.exceptions.GraphQLException('Expected "addTopupCard" to be present. Got: ' + str(response))
        
        if response['addTopupCard']['errors']:
            raise kard.exceptions.GraphQLException('Expected "addTopupCard.errors" to be empty. Got: ' + str(response))
        
        # Step 3: Return 3D secure form url
        return response['addTopupCard']['secureFormUrl']

        #~ TODO!

        



class InvalidPasscodeError(Exception):
    pass

class InvalidOTPError(Exception):
    pass

class InvalidPhoneNumberError(Exception):
    pass

class GraphQLException(Exception):
    pass

class CardNotFoundException(Exception):
    pass

class InsufficientFundsException(Exception):
    pass

class InvalidValueException(Exception):
    pass

class VaultNotFoundException(Exception):
    pass

class ContactNotFoundError(Exception):
    pass

class BeneficiaryNotFoundError(Exception):
    pass

class WireTransferInvalidAmount(Exception):
    pass



from ...app import Kard
from ... import kard

class KardTopupCard:
    def __init__(self, kard: Kard):
        self.app = kard

    def _new(self, cardId: str, **kwargs):
        card = KardTopupCard(self.app)
        card._setId(cardId)

        card._name = kwargs.get('name')
        card._expirationDate = kwargs.get('expirationDate')
        card._default = kwargs.get('default')
        card._last4 = kwargs.get('last4')
        card._providerSourceId = kwargs.get('providerSourceId')
        card._providerPaymentId = kwargs.get('providerPaymentId')

        return card

    def _setId(self, cardId: str):
        self._id = cardId
    
    def _getId(self):
        return self._id

    @property
    def id(self):
        return self._getId()

    @property
    def name(self):
        return self._name
    
    @property
    def expirationDate(self):
        return self._expirationDate
    
    @property
    def isDefault(self):
        return self._default
    
    @property
    def last4(self):
        return self._last4
    
    @property
    def providerSourceId(self):
        return self._providerSourceId
    
    @property
    def providerPaymentId(self):
        return self._providerPaymentId
    

    def forget(self):
        query = "mutation androidForgetTopupCard($topupCardId: ID!) { "\
                    "forgetTopupCard(input: { topupCardId: $topupCardId }) { "\
                        "errors { path message } "\
                    "}"\
                "}"
        variables = {
            "id": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('forgetTopupCard'):
            raise kard.exceptions.GraphQLException('Expected "forgetTopupCard" to be present. Got: ' + str(response))

        return response['forgetTopupCard']

    def delete(self):
        return self.forget()

    def setDefault(self):
        query = "mutation androidUpdateTopupCard($topupCardId: ID!) { "\
                    "updateTopupCard(input: {topupCardId: $topupCardId, attributes: {default: true}}) { "\
                        "errors { message path } "\
                    "}"\
                "}"
        variables = {
            "topupCardId": self.id
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('updateTopupCard'):
            raise kard.exceptions.GraphQLException('Expected "updateTopupCard" to be present. Got: ' + str(response))

        if response['updateTopupCard'].get('errors'):
            raise kard.exceptions.GraphQLException('Expected "updateTopupCard.errors" to be empty. Got: ' + str(response))

        return True

    def makeDefault(self):
        return self.setDefault()




import os

from . import graphql
from . import utils
from . import settings


class Kard:
    api_host = "api.kard.eu"

    def __init__(self, phoneNumber: str):
        self.phoneNumber = phoneNumber
        self.settings = settings.KardSettings(self.phoneNumber)

        self.platform = "android"
        self.vendorIdentifier = self.settings.getVendorIdentifier()
        if not self.vendorIdentifier:
            vI = utils.generateVendorIdentifier(platform=self.platform)
            self.vendorIdentifier = vI
            self.settings.setVendorIdentifier(self.vendorIdentifier)


        self.graphql = graphql.GraphQL(self.api_host, '/graphql', self.vendorIdentifier)

        self.karder = kard.karder.setup(self)


    def authenticate(self, forceApiAuth: bool=False):
        if not forceApiAuth:
            accessToken = self.settings.getAccessToken()
            if accessToken:
                self.graphql.setAccessToken(accessToken)
                return True

        query = "mutation androidInitSession($createUser: Boolean, $phoneNumber: PhoneNumber!,"\
                    " $platform: DevicePlatform, $vendorIdentifier: String!)"\
                "{ initSession(input: {createUser: $createUser, phoneNumber: $phoneNumber,"\
                    " platform: $platform, vendorIdentifier: $vendorIdentifier})"\
                " { challenge expiresAt errors { path message } }}"

        variables = {
            "platform": self.platform.upper(),
            "vendorIdentifier": self.vendorIdentifier,
            "phoneNumber": self.phoneNumber,
            "createUser": True
        }
        extensions = {}

        response = self.graphql.request(query, variables, extensions)
        if not response.get('initSession'):
            for error in response['errors']:
                if error['message'] == 'Variable $phoneNumber of type PhoneNumber! was provided invalid value':
                    raise kard.exceptions.InvalidPhoneNumberError("Invalid phone number %s" % self.phoneNumber)

        if response['initSession'].get('challenge') == 'OTP':
            return self.authenticateOTP()

        elif response['initSession'].get('challenge') == 'PASSCODE':
            return self.authenticatePasscode()

    def authenticateOTP(self):
        otp = input("Enter OTP: ")

        query = "mutation androidVerifyOTP($authenticationProvider: AuthenticationProviderInput, $code: String!,"\
                    " $phoneNumber: PhoneNumber!, $vendorIdentifier: String!)"\
                " { verifyOtp(input: {authenticationProvider: $authenticationProvider, code: $code, phoneNumber: $phoneNumber,"\
                    " vendorIdentifier: $vendorIdentifier})"\
                " { challenge accessToken refreshToken errors { path message } }}"
        variables = {
            "code": otp,
            "phoneNumber": self.phoneNumber,
            "vendorIdentifier": self.vendorIdentifier
        }
        extensions = {}

        response = self.graphql.request(query, variables, extensions)
        if not response.get('verifyOtp'): raise Exception("Invalid response %s" % response)

        if response['verifyOtp'].get('challenge') == 'PASSCODE':
            return self.authenticatePasscode()

        elif response['verifyOtp'].get('errors'):
            for error in response['verifyOtp']['errors']:
                if error['message'] == 'That code didn’t work':
                    raise kard.exceptions.InvalidOTPError("Invalid OTP %s" % otp)

    def authenticatePasscode(self):
        passcode = self.settings.getPasscode()
        if not passcode:
            passcode = input("Enter passcode: ")
            # Ask if user wants to save passcode?
            self.settings.setPasscode(passcode)


        query = "mutation androidSignIn($authenticationProvider: AuthenticationProviderInput,$passcode: String!,"\
                    " $phoneNumber: PhoneNumber!, $vendorIdentifier: String!)"\
                " { signIn(input: {authenticationProvider: $authenticationProvider,passcode: $passcode, phoneNumber: $phoneNumber,"\
                    " vendorIdentifier: $vendorIdentifier})"\
                " { accessToken refreshToken errors { path message } }}"
        variables = {
            "passcode": passcode,
            "phoneNumber": self.phoneNumber,
            "vendorIdentifier": self.vendorIdentifier
        }
        extensions = {}

        response = self.graphql.request(query, variables, extensions)
        if not response.get('signIn'): raise Exception("Invalid response %s" % response)

        if response['signIn'].get('accessToken'):
            self.settings.setAccessToken(response['signIn']['accessToken'])
            self.settings.setRefreshToken(response['signIn']['refreshToken'])
            
            self.graphql.setAccessToken(response['signIn']['accessToken'])
            return True

        elif response['signIn'].get('errors'):
            for error in response['signIn']['errors']:
                if error['message'] == 'Invalid passcode':
                    self.settings.setPasscode(None)
                    raise kard.exceptions.InvalidPasscodeError("Invalid passcode %s" % passcode)


from . import kard



from ...app import Kard
from ... import kard

class KardFamily:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidFamily { me { family { ... Family_FamilyParts } }}\n\n"\
    		"fragment Topup_TopupCardParts on TopupCard { "\
                "id name expirationDate default last4 providerSourceId providerPaymentId"\
            "}\n\n"\
			"fragment Topup_RecurringParts on RecurringPayment { "\
                "id active amount { value currency { symbol isoCode } }"\
                "child { id } firstPayment nextPayment cancelledAt topupCard { ... Topup_TopupCardParts }"\
            "}\n\n"\
			"fragment Card_CardParts on Card { __typename id activatedAt customText name visibleNumber blocked scheme"\
                "... on PhysicalCard { atm contactless swipe online design orderedAt }"\
            "}\n\n"\
			"fragment Me_KycParts on Kyc { required deadline globalStatus fundsOrigin { status value rejectionReason }"\
                " identityVerification { status url rejectionReason score } proofOfAddress {"\
                    " status files { contentType url ... on Image { width height } } rejectionReason "\
                "} canStart lastRejectedIdentityVerification { status url rejectionReason score} nbOfRejectedIdentityVerifications"\
            "}\n\n"\
			"fragment Me_SubscriptionParts on Subscription { id status cancelledAt cancellationReason nextBilling { "\
                "date amount { value currency { isoCode name symbol symbolFirst } } } plan {"\
                    " __typename id periodUnit name providerId price { value } }"\
            "}\n\n"\
			"fragment Family_MeParts on Me { id type createdAt profile { avatar { url } firstName lastName username age "\
                "birthday placeOfBirth shippingAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } "\
                "homeAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } } card { id } "\
                "canOrderCard cardBeingSetup { __typename id customText name ... on PhysicalCard { design } } "\
                "outgoingRecurringPayments { ... Topup_RecurringParts } incomingRecurringPayment { ... Topup_RecurringParts } "\
                "cards { nodes { ... Card_CardParts } } email claimId canOrderCard phoneNumber bankAccount { "\
                    "id iban bic user { firstName lastName } balance { value currency { symbol isoCode } } lockedTransactions } "\
                "topupCards { ... Topup_TopupCardParts } kyc { ... Me_KycParts } subscription { ... Me_SubscriptionParts } "\
                "onboardingDone savingsAmount { value } topupIncentiveActivated noPhoneNumber currentSpendings { "\
                    "monthlyPos { value } weeklyPos { value } weeklyAtm { value } monthlyAtm { value } "\
                "} transactionLimits { id amount { value currency { symbol } } recurrence transactionType }"\
            "}\n\n"\
			"fragment Family_FamilyParts on Family { memberships { status primary type nickname member { ... Family_MeParts } }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('family', {}):
            raise kard.exceptions.GraphQLException('Expected "me.family" to be present. Got: ' + str(response))

        return response

    def getMembers(self):
        query = "query androidFamily { me { family { ... Family_FamilyParts } }}\n\n"\
            "fragment Family_MeParts on Me { id type createdAt profile { avatar { url } firstName lastName username age "\
                "birthday placeOfBirth shippingAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } "\
                "homeAddress { firstName lastName street line1 line2 zipcode city state country fullAddress } } "\
                "email claimId phoneNumber "\
            "}\n\n"\
			"fragment Family_FamilyParts on Family { memberships { status primary type nickname member { ... Family_MeParts } }}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('family', {}):
            raise kard.exceptions.GraphQLException('Expected "me.family" to be present. Got: ' + str(response))

        return response

    def getSiblings(self):
        members = self.getMembers()
        siblings = []
        for member in members['me']['family']['memberships']:
            if member['member']['id'] != self.app.details.id:
                if member['type'] != 'PARENT':
                    siblings.append(member)

        return siblings

    def getParents(self):
        members = self.getMembers()
        parents = []
        for member in members['me']['family']['memberships']:
            if member['member']['id'] != self.app.details.id:
                if member['type'] == 'PARENT':
                    parents.append(member)

        return parents

    def getChildren(self):
        # Siblings are anyone but parents, so if user is a parent, siblings returned are in fact children
        return self.getSiblings()



import uuid

def generateUUID():
    """ Generate a UUID, and return it. """
    return str(uuid.uuid4())

def generateVendorIdentifier(platform: str):
    """ Generate a vendor identifier for the platform, and return it. """
    vI = "%s:%s" % (platform, generateUUID())
    return vI




from ...app import Kard
from ... import kard

class KardOffers:
    def __init__(self, kard: Kard):
        self.app = kard

    def getCompleteData(self):
        query = "query androidOffers { "\
            "cashbackOffers { "\
                "name url cashbackRate channel description legalTerms isConsumed maxPerUser brand { name logoUrl description } "\
                "minAmount maxAmount pictureUrl startDate endDate "\
            "} "\
        "}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('cashbackOffers'):
            raise kard.exceptions.GraphQLException('Expected "cashbackOffers" to be present. Got: ' + str(response))

        return response['cashbackOffers']

    def get(self):
        return self.getCompleteData()


from ...app import Kard
from ... import kard

class KardTransactions:
    def __init__(self, kard: Kard):
        self.app = kard

    def get(self, first: int=20, after: str=None, numberOfComments: int=3):
        query = "query androidTransactions($first: Int, $after: String, $numberOfComments: Int) { me { "\
                    "bankAccount { balance { value } } "\
                    "typedTransactions(first: $first, after: $after) { "\
                        "pageInfo { endCursor hasNextPage } "\
                        "nodes { "\
                            "__typename id title status visibility amount { value currency { symbol } } "\
                            "category { name color image { url } } user { id firstName lastName username } "\
                            "processedAt unreadCommentsCount lastCommentReadAt "\
                            "comments(first: $numberOfComments) { "\
                                "totalCount pageInfo { endCursor hasNextPage } "\
                                "nodes { id comment createdAt user { id nickname firstName lastName avatar { url } } } "\
                            "} "\
                            "...on P2pTransaction { triggeredBy { id firstName lastName username avatar { url } } reason } "\
                            "...on OutgoingWireTransferTransaction { "\
                                "beneficiary { bankLogo { url } bankName iban bic id surname fullName } reason "\
                            "} "\
                            "...on ClosingAccountTransaction { moneyAccount { ... Vault_VaultMiniParts } } "\
                            "...on CardTransaction { "\
                                "card { ... Card_CardParts } mobilePaymentProvider "\
                                "cashback { id status brandLogo brandName amount { value currency { symbol } } } "\
                            "} "\
                            "...on InternalTransferTransaction { moneyAccount { ... Vault_VaultMiniParts } } "\
                            "... on MoneyLinkTransaction { from message } "\
                            "... on TopupTransaction { sender { nickname member { id profile { firstName } } } message } "\
                            "... on RejectedTransaction { rejectionReason } "\
                            "... on CashbackTransaction { cashback { id status brandLogo brandName amount { value currency { symbol } } "\
                                "sourceTransaction { "\
                                    "id title image { id url } category { name color image { url } } "\
                                    "amount { value currency { symbol } } } "\
                                "} "\
                            "} "\
                            "...on AtmTransaction { card { ... Card_CardParts } } "\
                        "} "\
                    "} "\
                "}}\n\n"\
                "fragment Vault_VaultMiniParts on Vault { name color emoji { name unicode }}\n\n"\
                "fragment Card_CardParts on Card { "\
                    "__typename id activatedAt customText name visibleNumber blocked expirationAt "\
                    "scheme ... on PhysicalCard { "\
                        "atm contactless swipe online design reorderExpiredFees { value currency { isoCode } } "\
                        "orderedAt orderDelayed isMain isSoonExpired estimatedCraftingDate estimatedDeliveryDate "\
                    "}"\
                "}"
        variables = {
            'first': first,
            'after': after,
            'numberOfComments': numberOfComments
        }
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('me', {}).get('typedTransactions'):
            raise kard.exceptions.GraphQLException('Expected "me.typedTransactions" to be present. Got: ' + str(response))
        
        return response['me']['typedTransactions']

    def getAll(self):
        transactions = []
        lastPageInfo = {}

        x = self.get()
        transactions.append(x['nodes'])
        lastPageInfo = x['pageInfo']

        while lastPageInfo['hasNextPage']:
            x = self.get(after=lastPageInfo['endCursor'])
            transactions.append(x['nodes'])
            lastPageInfo = x['pageInfo']

        return transactions




import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()


setuptools.setup(
    name="pyKard",
    version="3.0.1",
    author="Gaëtan Hérault",
    author_email="dev@ghr.lt",
    description="Python wrapper for Kard's private API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ghrlt/py-kard",
    project_urls={
    	"Developer website": "https://ghr.lt",
        "Bug Tracker": "https://github.com/ghrlt/py-kard/issues",
    },
    install_requires=[
        "requests"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    packages=["kard_private_api", "kard_private_api.kard", "kard_private_api.kard.karder"],
    python_requires=">=3.6",
)


from ...app import Kard
from ... import kard

class KardNotifications:
    def __init__(self, kard: Kard):
        self.app = kard


    def getCompleteData(self):
        query = "query androidGetActivityItems { activityItems { "\
                    "id onClickUrl title body icon isRead isDone createdAt device modal "\
                        "{ title body buttonTitle onClickUrl kardy withCelebration } "\
                "}}"
        variables = {}
        extensions = {}

        response = self.app.graphql.request(query, variables, extensions)
        if not response.get('activityItems'):
            raise kard.exceptions.GraphQLException('Expected "activityItems" to be present. Got: ' + str(response))

        return response['activityItems']



import os
import json

class KardSettings:
    def __init__(self, phone_number: str):
        self.phone_number = phone_number
        self.file = ".kard-login_%s-settings.json" % self.phone_number


        if os.name == 'nt':
            self.file_path = os.path.join('C:\\', 'Users', os.getlogin(), self.file)
        else:
            self.file_path = os.path.join(os.path.expanduser('~'), self.file)


    def getSettings(self):
        if not os.path.exists(self.file_path):
            self.saveSettings({
                "phoneNumber": self.phone_number
            })

        with open(self.file_path, 'r', encoding='utf-8') as f:
            return json.load(f)

    def saveSettings(self, settings: dict):
        with open(self.file_path, 'w', encoding='utf-8') as f:
            json.dump(settings, f, ensure_ascii=False, indent=2)


    def getVendorIdentifier(self):
        return self.getSettings().get("vendorIdentifier")

    def setVendorIdentifier(self, vendorIdentifier: str|None):
        settings = self.getSettings()
        settings["vendorIdentifier"] = vendorIdentifier
        
        if vendorIdentifier is None:
            del settings['vendorIdentifier']
        
        self.saveSettings(settings)

    def getPlatform(self):
        return self.getVendorIdentifier().split(":")[0]

    def getPasscode(self):
        return self.getSettings().get("passcode")
    
    def setPasscode(self, passcode: str|None):
        settings = self.getSettings()
        settings["passcode"] = passcode
        
        if passcode is None:
            del settings['passcode']
        
        self.saveSettings(settings)

    def getAccessToken(self):
        return self.getSettings().get("accessToken")
    
    def setAccessToken(self, accessToken: str|None):
        settings = self.getSettings()
        settings["accessToken"] = accessToken
        
        if accessToken is None:
            del settings['accessToken']
        
        self.saveSettings(settings)
    
    def getRefreshToken(self):
        return self.getSettings().get("refreshToken")
    
    def setRefreshToken(self, refreshToken: str|None):
        settings = self.getSettings()
        settings["refreshToken"] = refreshToken
        
        if refreshToken is None:
            del settings['refreshToken']
        
        self.saveSettings(settings)

